precision mediump float;
uniform sampler2D inputImageTexture;
varying vec2 v_texcoord;

// Constants
const mediump vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);
const highp vec4 kRGBToYPrime = vec4 (0.299, 0.587, 0.114, 0.0);
const highp vec4 kRGBToI = vec4 (0.595716, -0.274453, -0.321263, 0.0);
const highp vec4 kRGBToQ = vec4 (0.211456, -0.522591, 0.31135, 0.0);
const highp vec4 kYIQToR = vec4 (1.0, 0.9563, 0.6210, 0.0);
const highp vec4 kYIQToG = vec4 (1.0, -0.2721, -0.6474, 0.0);
const highp vec4 kYIQToB = vec4 (1.0, -1.1070, 1.7046, 0.0);
const lowp vec3 warmFilter = vec3(0.93, 0.54, 0.0);
const highp mat3 RGBtoYIQ = mat3(0.299, 0.587, 0.114, 0.595716, -0.274453, -0.321263, 0.211456, -0.522591, 0.31135);
const highp mat3 YIQtoRGB = mat3(1.0, 0.9563, 0.621, 1.0, -0.2721, -0.647, 1.0, -1.1070, 1.7046);
const int number_of_filters = 10;
uniform int filters_l[number_of_filters];
uniform int filters_r[number_of_filters];

// Left side filter settings
uniform lowp float saturation_l;
uniform lowp float brightness_l;
uniform lowp float contrast_l;
uniform mediump float hueAdjust_l;
uniform lowp float redMin_l;
uniform lowp float redMax_l;
uniform lowp float greenMin_l;
uniform lowp float greenMax_l;
uniform lowp float blueMin_l;
uniform lowp float blueMax_l;
uniform lowp float gamma_l;
uniform lowp mat4 colorMatrix_l;
uniform lowp float intensity_l;
uniform highp float exposure_l;
uniform lowp float temperature_l;
uniform lowp float tint_l;

// Right side filter settings
uniform lowp float saturation_r;
uniform lowp float brightness_r;
uniform lowp float contrast_r;
uniform mediump float hueAdjust_r;
uniform lowp float redMin_r;
uniform lowp float redMax_r;
uniform lowp float greenMin_r;
uniform lowp float greenMax_r;
uniform lowp float blueMin_r;
uniform lowp float blueMax_r;
uniform lowp float gamma_r;
uniform lowp mat4 colorMatrix_r;
uniform lowp float intensity_r;
uniform highp float exposure_r;
uniform lowp float temperature_r;
uniform lowp float tint_r;

uniform lowp float position;


vec4 saturation(vec4 textureColor, float saturation) {
   lowp float luminance = dot(textureColor.rgb, luminanceWeighting);
   lowp vec3 greyScaleColor = vec3(luminance);
   textureColor = vec4(mix(greyScaleColor, textureColor.rgb, saturation), textureColor.w);
   return textureColor;
}

vec4 brightness(vec4 textureColor, float brightness) {
   textureColor = vec4((textureColor.rgb + vec3(brightness)), textureColor.w);
   return textureColor;
}

vec4 contrast(vec4 textureColor, float contrast) {
   textureColor = vec4(((textureColor.rgb - vec3(0.5)) * contrast + vec3(0.5)), textureColor.w);
   return textureColor;
}

vec4 hue(vec4 textureColor, float hueAdjust) {

   // Convert to YIQ
   highp float YPrime = dot (textureColor, kRGBToYPrime);
   highp float I = dot (textureColor, kRGBToI);
   highp float Q = dot (textureColor, kRGBToQ);

   // Calculate the hue and chroma
   highp float hue = atan (Q, I);
   highp float chroma = sqrt (I * I + Q * Q);

   // Make the user's adjustments
   hue += (-hueAdjust);

   // Convert back to YIQ
   Q = chroma * sin (hue);
   I = chroma * cos (hue);

    // Convert back to RGB
   highp vec4 yIQ = vec4 (YPrime, I, Q, 0.0);

   textureColor.r = dot (yIQ, kYIQToR);
   textureColor.g = dot (yIQ, kYIQToG);
   textureColor.b = dot (yIQ, kYIQToB);
   return textureColor;
}

vec4 colorClamp(vec4 textureColor,
           float redMin, float redMax,
           float greenMin, float greenMax,
           float blueMin, float blueMax) {
    textureColor.r = max(textureColor.r,redMin);
    textureColor.r = min(textureColor.r,redMax);
    textureColor.g = max(textureColor.g,greenMin);
    textureColor.g = min(textureColor.g,greenMax);
    textureColor.b = max(textureColor.b,blueMin);
    textureColor.b = min(textureColor.b,blueMax);
   return textureColor;
}
vec4 gamma(vec4 textureColor, float gamma) {
   textureColor = vec4(pow(textureColor.rgb, vec3(gamma)), textureColor.w);
   return textureColor;
}

vec4 colorMatrix(vec4 textureColor, mat4 matrix, float intensity) {
   textureColor = (intensity * (textureColor * matrix)) + ((1.0 - intensity) * textureColor);
   return textureColor;
}

vec4 exposure(vec4 textureColor, float exposure) {
   textureColor = vec4(textureColor.rgb * pow(2.0, exposure), textureColor.w);
   return textureColor;
}

vec4 tempAndTint(vec4 textureColor, float tint, float temperature) {
   mediump vec3 yiq = RGBtoYIQ * textureColor.rgb; //adjusting tint
   yiq.b = clamp(yiq.b + tint*0.5226*0.1, -0.5226, 0.5226);
   lowp vec3 rgb = YIQtoRGB * yiq;

   lowp vec3 processed = vec3(
    (rgb.r < 0.5 ? (2.0 * rgb.r * warmFilter.r) : (1.0 - 2.0 * (1.0 - rgb.r) * (1.0 - warmFilter.r))),
    (rgb.g < 0.5 ? (2.0 * rgb.g * warmFilter.g) : (1.0 - 2.0 * (1.0 - rgb.g) * (1.0 - warmFilter.g))),
    (rgb.b < 0.5 ? (2.0 * rgb.b * warmFilter.b) : (1.0 - 2.0 * (1.0 - rgb.b) * (1.0 - warmFilter.b))));

   lowp float tempInterpolation = temperature < 5000.0 ? 0.0004 * (temperature-5000.0) : 0.00006 * (temperature-5000.0);
   return vec4(mix(rgb, processed, tempInterpolation), textureColor.a);
}

void main() {
    lowp vec4 textureColor = texture2D(inputImageTexture, v_texcoord);
    if (v_texcoord.x < position)
    {
       for(int i = 0; i < number_of_filters; i = i + 1 ){
           if(filters_l[i] == 1){
               textureColor = saturation(textureColor, saturation_l);
           }
           if(filters_l[i] == 2){
               textureColor = brightness(textureColor, brightness_l);
           }
           if(filters_l[i] == 3){
               textureColor = contrast(textureColor, contrast_l);
           }
           if(filters_l[i] == 4){
               textureColor = hue(textureColor, hueAdjust_l);
           }
           if(filters_l[i] == 5){
               textureColor = colorClamp(textureColor, redMin_l, redMax_l, greenMin_l, greenMax_l, blueMin_l, blueMax_l);
           }
           if(filters_l[i] == 6){
               textureColor = gamma(textureColor, gamma_l);
           }
           if(filters_l[i] == 7){
               textureColor = colorMatrix(textureColor, colorMatrix_l, intensity_l);
           }
           if(filters_l[i] == 8){
               textureColor = exposure(textureColor, exposure_l);
           }
           if(filters_l[i] == 9){
               textureColor = tempAndTint(textureColor, tint_l, temperature_l);
           }
       }
    }
    else
    {
       for(int i = 0; i < number_of_filters; i = i + 1 ){
           if(filters_r[i] == 1){
               textureColor = saturation(textureColor, saturation_r);
           }
           if(filters_r[i] == 2){
               textureColor = brightness(textureColor, brightness_r);
           }
           if(filters_r[i] == 3){
               textureColor = contrast(textureColor, contrast_r);
           }
           if(filters_r[i] == 4){
               textureColor = hue(textureColor, hueAdjust_r);
           }
           if(filters_r[i] == 5){
               textureColor = colorClamp(textureColor, redMin_r, redMax_r, greenMin_r, greenMax_r, blueMin_r, blueMax_r);
           }
           if(filters_r[i] == 6){
               textureColor = gamma(textureColor, gamma_r);
           }
           if(filters_r[i] == 7){
               textureColor = colorMatrix(textureColor, colorMatrix_r, intensity_r);
           }
           if(filters_r[i] == 8){
               textureColor = exposure(textureColor, exposure_r);
           }
           if(filters_r[i] == 9){
               textureColor = tempAndTint(textureColor, tint_r, temperature_r);
           }
       }
    }
    gl_FragColor = textureColor;
}
